<?php

namespace Drupal\simple_analytics\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\simple_analytics\SimpleAnalyticsHelper;
use Drupal\Core\Database\Database;
use Drupal\simple_analytics\SimpleAnalyticsService;

/**
 * Simple Analytics Functions.
 */
class SimpleAnalytics extends ControllerBase {

  /**
   * Display today visitors details.
   */
  public function viewToday() {
    $output = [];
    $con = Database::getConnection();
    $output[] = ['#markup' => "<h2> Today statistic at : " . date("H:i:s") . "</h2>"];
    $fields = [
      'id',
      'timestamp',
      'REFERER',
      'MOBILE',
      'SCREEN',
      'REMOTE_ADDR',
      'HTTP_REFERER',
      'LANGUAGE',
      'BOT',
      'SIGNATURE',
    ];
    $fields_disp = [
      'id' => "#",
      'timestamp' => "First",
      // Virtual field.
      'count' => "Count",
      // Virtual field.
      'visit_last' => "Last visit",
      'REFERER' => "Referer address",
      'SCREEN' => "Resolution",
      'REMOTE_ADDR' => "IP Address",
      'LANGUAGE' => "⚑",
      'BOT' => '☢',
      'MOBILE' => '☎',
    ];
    $html = "";
    $limit = 10000;
    $query = $con->select('simple_analytics_data', 'd');
    $query->fields('d', $fields);
    $query->orderBy("id", 'DESC');
    // LIMIT.
    $query->range(0, $limit * 10);
    $result = $query->execute()->fetchAll();
    $visits = count($result);
    $page_view = 0;
    $mobile = 0;
    $entry_direct = 0;
    $bot = 0;
    $languages = [];

    // Prepare Table.
    foreach ($result as $row) {
      $page_view++;
      $mobile += $row->MOBILE;
      $bot += $row->BOT;
      $entry_direct += ($row->REFERER ? 0 : 1);
      if (isset($languages[$row->LANGUAGE])) {
        $languages[$row->LANGUAGE]++;
      }
      else {
        $languages[$row->LANGUAGE] = 1;
      }
    }

    $html .= "Total visits : $visits (Mobile:$mobile, Bots:$bot)<br>Total Page view : $page_view<br>";

    $lables = [];
    $datas = [];

    foreach ($fields_disp as $field) {
      $lables[] = $field;
    }

    foreach ($result as $index => $row) {
      $signature = $row->SIGNATURE;

      if (isset($datas[$signature])) {
        $data_row = $datas[$signature];
      }

      foreach ($fields_disp as $field => $field_label) {

        $value = $row->$field;
        if (strstr($field, 'timestamp') !== FALSE) {
          $value = isset($datas[$signature][$field]) ? $datas[$signature][$field] : date("H:i:s", $value);
        }
        elseif ($field == 'REFERER' && $value) {
          $value = parse_url($value, PHP_URL_HOST);
        }
        elseif ($field == 'count') {
          $value = $datas[$signature][$field] ? $datas[$signature][$field] + 1 : 1;
        }
        elseif ($field == 'visit_last') {
          $value = date("H:i:s", $row->timestamp);
        }
        elseif ($field == 'BOT' || $field == 'MOBILE') {
          $value = $value ? '✔' : '';
        }

        $data_row[$field] = $value;
      }
      if ($index > $limit) {
        break;
      }

      $datas[$signature] = $data_row;
    }

    $output[] = ['#markup' => $html];

    $output['#attached']['library'][] = 'simple_analytics/simple_analytics_main';
    $output[] = [
      '#theme' => 'table',
      '#rows' => $datas,
      '#header' => $lables,
      '#empty' => '',
    ];

    $output[] = ['#markup' => "<p>☢ : Bots, ☎ : Mobiles, ⚑ : Languages</p>"];
    return $output;
  }

  /**
   * Display visitors settings.
   */
  public function viewSettings() {
    $output = [];

    $con = Database::getConnection();
    $config = SimpleAnalyticsHelper::getConfig();

    // Days.
    $limit = $config->get('displaystat');

    $output[] = ['#markup' => "<h2>Visiteurs Settings (last $limit Days)</h2>"];

    $query = $con->select('simple_analytics_archive', 'd');
    $query->fields('d', [
      'id',
      'date',
      'visits',
      'page_view',
      'mobile',
      'entry_direct',
      'bots',
      'settings_data',
    ]);
    $query->orderBy("date", 'ASC');
    // LIMIT.
    $query->range(0, $limit);
    $result = $query->execute()->fetchAll();
    $visits = 0;
    $page_view = 0;
    $temp['screens'] = [];
    $temp['languages'] = [];

    foreach ($result as $row) {
      $page_view += $row->page_view;
      $visits += $row->visits;

      // Get settings data array.
      $settings_data = json_decode($row->settings_data, TRUE);
      $screens = $settings_data['screens'];
      if (is_array($screens)) {
        foreach ($screens as $size => $count) {
          if (isset($temp['screens'][$size])) {
            $temp['screens'][$size] += $count;
          }
          else {
            $temp['screens'][$size] = 1;
          }
        }
      }

      $languages = $settings_data['languages'];
      if (is_array($languages)) {
        foreach ($languages as $size => $count) {
          if (isset($temp['languages'][$size])) {
            $temp['languages'][$size] += $count;
          }
          else {
            $temp['languages'][$size] = 1;
          }
        }
      }
    }

    // Summary.
    $html = "";
    $html .= "Total visits : $visits<br>Total Page view : $page_view<br>";
    $output[] = [
      '#markup' => $html,
    ];

    // Table Languages.
    $lables = [];
    $datas = [];
    $count_row = 0;
    // Sort by count DESC.
    arsort($temp['languages']);
    foreach ($temp['languages'] as $label => $count) {
      if (!$label || $label == "_empty_") {
        $label = "Unknown";
      }
      $lables[] = $label;
      $datas[0][] = $count;
      $count_row++;
      if ($count_row >= 10) {
        break;
      }
    }
    $fields = ['Languages', 'Count'];
    $output[] = SimpleAnalyticsService::generateTableVertical($lables, $datas, [
      'title' => 'Languages',
      'fields' => $fields,
    ]);

    // Table Screen resolution.
    $lables = [];
    $datas = [];
    $count_row = 0;
    // Sort by count DESC.
    arsort($temp['screens']);
    foreach ($temp['screens'] as $label => $count) {
      if (!$label || $label == "_empty_") {
        $label = "Unknown";
      }
      $lables[] = $label;
      $datas[0][] = $count;
      $count_row++;
      if ($count_row >= 10) {
        break;
      }
    }
    $fields = ['Resolution', 'Count'];
    $output[] = SimpleAnalyticsService::generateTableVertical($lables, $datas, [
      'title' => 'Screen resolutions',
      'fields' => $fields,
    ]);

    return $output;
  }

  /**
   * Display visitors history.
   */
  public function viewHistory($filter = NULL) {

    $config = SimpleAnalyticsHelper::getConfig();

    $output = [];
    // No cache.
    $output[]['#cache']['max-age'] = 0;
    $con = Database::getConnection();
    $config = SimpleAnalyticsHelper::getConfig();

    // Days.
    $limit = $config->get('displaystat');

    $fields = [
      'id',
      'date',
      'visits',
      'page_view',
      'mobile',
      'entry_direct',
      'bots',
      'hits',
      'bots_hits',
      'settings_data',
    ];

    // Authorized fields and defaults.
    $fields_auth = [
      'visits' => 1,
      'page_view' => 1,
      'mobile' => 1,
      'entry_direct' => 0,
      'bots' => 1,
      'hits' => 1,
      'bots_hits' => 0,
    ];
    $fields_legends = [
      'visits' => 'Visits',
      'page_view' => 'View',
      'bots' => 'Bots',
      'mobile' => 'Mobiles',
      'hits' => 'Total Pages',
      'entry_direct' => 'Direct entry',
      'bots_hits' => 'Bots Hits',
    ];
    $filter_str = $filter ? (" | " . $fields_legends[$filter]) : '';
    $output[] = ['#markup' => "<h2> History of last $limit Days $filter_str</h2>"];

    // Remove unwanted filters.
    if ($filter && !isset($fields_auth[$filter])) {
      $filter = NULL;
    }
    $fields_display = [];
    $chart_legends = [];

    if ($filter) {
      // Filtered single view.
      $fields_display[] = $filter;
      $chart_legends[] = $fields_legends[$filter];
    }
    else {
      // Default View.
      foreach ($fields_auth as $field => $is_default) {
        if ($is_default) {
          $fields_display[] = $field;
          $chart_legends[] = $fields_legends[$field];
        }
      }
    }

    $query = $con->select('simple_analytics_archive', 'd');
    $query->fields('d', $fields);
    $query->orderBy("date", 'DESC');
    // LIMIT.
    $query->range(0, $limit);
    $result = $query->execute()->fetchAll();
    $result = array_reverse($result);

    // Display chart.
    if ($config->get('lib_chartist')) {
      $lables = [];
      $datas = [];

      $cpt = 0;

      foreach ($result as $row) {
        $i = 0;
        $lables[] = date("M-d", $row->date);
        foreach ($fields_display as $field) {
          $datas[$i++][] = $row->$field;
        }
        $cpt++;
      }
      $data = SimpleAnalyticsService::generateGraph($lables, $datas, [], $output);

      $output[] = [
        '#theme' => 'simple_analytics_chart_01',
        '#chart_title' => 'The title',
        '#chart_type' => $data['chart_type'],
        '#chart_id' => $data['chart_id'],
        '#chart_labels' => $data['chart_labels'],
        '#chart_series' => $data['chart_series'],
        '#chart_legends' => $chart_legends,
      ];
    }

    // Build table.
    $lables = ['date' => 'Date'];
    $data = [];

    $fields = $fields_legends;
    if ($filter) {
      // Filtered single view.
      $fields = [$filter => $fields_legends[$filter]];
      $lables[$filter] = $fields_legends[$filter];
    }
    else {
      $lables = array_merge($lables, $fields_legends);
    }

    foreach ($result as $key => $row) {
      $data[$key] = [
        'date' => date("Y-m-d", $row->date),
      ];

      foreach ($fields as $field => $field_Name) {
        $data[$key][$field] = $row->$field;
      }
    }

    // Revert data order.
    rsort($data);
    $output[] = [
      '#theme' => 'table',
      '#rows' => $data,
      '#header' => $lables,
      '#empty' => '',
    ];

    return $output;
  }

}
